from logging import Logger, getLogger
from typing import Callable, Union

from httpx import HTTPStatusError
from tenacity import RetryCallState


def get_logger(name: str, parent: Union[Logger, None] = None) -> Logger:
    if parent:
        return parent.getChild(name)

    return getLogger(f"vtex.{name}")


def log_before_retry(
    logger: Logger,
    log_level: int,
) -> Callable[[RetryCallState], None]:
    def retry_log(retry_state: RetryCallState) -> None:
        if not retry_state.outcome or not retry_state.next_action:
            raise RuntimeError("Retry log called before request was finished")

        exception = retry_state.outcome.exception()
        if not isinstance(exception, HTTPStatusError):
            raise RuntimeError("Retry log called without an http status error outcome")

        method = str(exception.request.method)
        url = str(exception.request.url)
        status = str(exception.response.status_code)
        reason = str(exception.response.reason_phrase)

        logger.log(
            log_level,
            f"Retrying {method} {url} in {retry_state.next_action.sleep}s as "
            f"attempt {retry_state.attempt_number} failed with: {status} - {reason}",
            extra={
                "exception": exception,
                "request": exception.request.__dict__,
                "response": exception.response.__dict__,
                "retry_state": retry_state.__dict__,
            },
        )

    return retry_log


CLIENT_LOGGER = get_logger(name="client")
EXCEPTIONS_LOGGER = get_logger(name="exceptions")
UTILS_LOGGER = get_logger(name="utils")



from typing import Dict, Tuple, Type, Union


class Sentinel:
    _instances: Dict[str, "Sentinel"] = {}
    _name: str

    def __new__(cls, name: Union[str, None] = None) -> "Sentinel":
        _singleton_name = getattr(cls, "_singleton_name", None)

        if _singleton_name and name:
            raise ValueError("A Sentinel with _singleton_name can't receive a new name")

        name = _singleton_name or name or cls.__name__
        if name not in cls._instances:
            instance = super(Sentinel, cls).__new__(cls)
            instance._name = name
            cls._instances[name] = instance

        return cls._instances[name]

    def __reduce__(self) -> Tuple[Type["Sentinel"], Tuple[str]]:
        return self.__class__, (self._name,)

    def __hash__(self) -> int:
        return id(self)

    def __eq__(self, other: object) -> bool:
        return self is other

    def __repr__(self) -> str:
        return f"<Sentinel: {self._name}>"


class UndefinedSentinel(Sentinel):
    _default_name = "UNDEFINED"

    def __bool__(self) -> bool:
        return False


UNDEFINED = UndefinedSentinel()



from typing import Any, Dict, Literal

OrderingDirectionType = Literal["ASC", "DESC", "asc", "desc"]

HTTPMethodType = Literal[
    "DELETE",
    "GET",
    "HEAD",
    "OPTIONS",
    "PATCH",
    "POST",
    "PUT",
    "delete",
    "get",
    "head",
    "options",
    "patch",
    "post",
    "put",
]

DictType = Dict[str, Any]



from typing import TypedDict, Union


class GetAccountData(TypedDict, total=False):
    account_name: str
    company_name: str
    creation_date: str
    have_parent_account: bool
    id: str
    inactivation_date: Union[str, None]
    is_active: bool
    is_operating: bool
    name: str
    operation_date: Union[str, None]
    parent_account_id: Union[str, None]
    parent_account_name: Union[str, None]
    trading_name: str
